import os

from durant.exceptions import ConfigError

try:
    import configparser
except ImportError:
    import ConfigParser as configparser

DURANT_FILE = 'durant.conf'
DURANT_DIR = '.durant/'
DURANT_RELEASES_DIR = DURANT_DIR + 'releases/'


class Config(object):
    project_path = None
    config_path = None
    config = None

    def __init__(self, project_path=None):
        if not project_path:
            self.project_path = os.getcwd()

        self.config_path = os.path.realpath(
            os.path.join(self.project_path, DURANT_FILE))

        if not os.path.exists(self.config_path):
            raise ConfigError("Config file '%s' not found" % DURANT_FILE)

        try:
            self.config = configparser.ConfigParser()
            self.config.read_file(open(self.config_path))
        except:
            self.config = configparser.SafeConfigParser()
            self.config.readfp(open(self.config_path))

    def get(self, section, key, many=False, fallback=None):
        try:
            raw_value = self.config.get(section, key)
        except configparser.NoOptionError:
            raw_value = fallback

        if raw_value and many:
            value = raw_value.splitlines(
            ) if '\n' in raw_value else raw_value.split(',')
            return filter(None, map(str.strip, value))

        return raw_value

    def has_stage(self, stage):
        return self.config.has_section(stage)



ESC = '\033'
CSI = ESC + '['
OSC = ESC + ']'

ansi = {
    'Reset':      '0',
    'Black':      '0;30',
    'Red':        '0;31',
    'Green':      '0;32',
    'Yellow':     '0;33',
    'Blue':       '0;34',
    'Purple':     '0;35',
    'Cyan':       '0;36',
    'LightGray':  '0;37',
    'DarkGray':   '1;30', 
    'BoldRed':    '1;31',
    'BoldGreen':  '1;32',
    'BoldYellow': '1;33',
    'BoldBlue':   '1;34',
    'BoldPurple': '1;35',
    'BoldCyan':   '1;36',
    'White':      '1;37'
}

class Codes(object):
    def __init__(self):
        for name in ansi:
            setattr(self, name.upper(), CSI + ansi[name] + 'm')                                        

colors = Codes()



from __future__ import print_function

import os
import time

from colors import colors


def output(text, color=None, reset=True):
    output = (color if color else '') + text
    if reset:
        output += colors.RESET

    return output


def output_dots(text, char='.', end=None, size=None, color=None, reset=True):
    if not size:
        size = int(get_console_size()[-1])

    chars = char * (size - len(text))
    if end:
        chars = chars[len(end):]

    return output(text + chars, color, reset)


def get_console_size():
    rows, columns = os.popen('stty size', 'r').read().split()
    return (rows, columns)


class Console(object):
    def log(self, text, color=None, prefix=True, end='\n'):
        if prefix:
            date = output('[' + time.strftime('%X') + '] ',
                          color=colors.YELLOW)
            text = date + text
            
        print(output(text, color), end=end)

    def done(self, text='DONE', end='\n'):
        return self.log(text, color=colors.GREEN, prefix=False, end=end)

    def fail(self, text='FAIL', end='\n'):
        return self.log(text, color=colors.RED, prefix=False, end=end)

    def error(self, text, prefix='ERROR: ', end='\n'):
        err = output(prefix, color=colors.RED)
        return self.log(err + text)

    def nl(self):
        return self.log('', prefix=False)


console = Console()



# -*- coding: utf-8 -*-
"""Durant. Simple git deployment tool.

Usage examples:
- Deploy project to stage    
  $ durant deploy [-n] <stage>

Please run 'durant --help' to see full help"""

__title__ = 'durant'
__version__ = '0.2.3'
__author__ = 'Marius Bărbulescu'
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2015 Marius Bărbulescu'

import commands

from .config import Config
from .deployer import Deployer



from __future__ import print_function

import sys
import time
import argparse
import durant

from durant.cli import output
from durant.cli import console


def version():
    return 'Version %s' % durant.__version__


def main():
    parser = argparse.ArgumentParser(description='Durant. Simple git deployment tool.',
        epilog="Use '%(prog)s <command> --help' to get detailed usage for that particular command")
    parser.add_argument('-v', '--version', action='version', version=version())

    subparsers = parser.add_subparsers(dest='command', title='available commands')

    deploy_parser = subparsers.add_parser('deploy', help='Deploy to specified stage')
    deploy_parser.add_argument('-n', '--dry-run', action='store_true', default=False,
        help='Perform a trial run, without actually deploying')
    deploy_parser.add_argument('stage', action='store',
        help='Stage to deploy to (a valid section name defined in the config file)')

    args = parser.parse_args()
    
    if args.command == 'deploy':
        console.log('Starting deployment\n')

        try:
            console.log('Checking config file...', end='')
            config = durant.Config()
            console.done()
        except Exception as e:
            console.fail()
            sys.exit(1)

        start = time.time()    

        try:            
            cmd = durant.commands.Deploy(config)        
            cmd.execute(stage=args.stage, dry_run=args.dry_run)

            deployed = True
        except Exception as e:
            console.error(str(e))
            deployed = False

        end = time.time()

        console.nl()
        console.log('Deployment %s (%.3f seconds)' %
            ('complete' if deployed else 'failed', (end - start)))

        if not deployed:
            sys.exit(1)


if __name__ == "__main__":
    main()
